import pygame, sys, strategy, string, random
import pygame._view
import mechanics, time
from constants import *

#Set mode flags
DEBUG= "-d" in sys.argv or "-debug" in sys.argv
WORK= "-w" in sys.argv or "-work" in sys.argv
SECRETS=0

#An object to represent a right-click popup menu
class Menu:
  width= 200
  height=325
  color=""
  country=""
  def __init__(self, color, country, x, y):
    self.color= color
    self.country=country
    self.x, self.y= x, y
    if self.x+self.width > width: self.x= self.x-self.width
    if self.y+self.height > height: self.y= self.y-self.height
  
#Begin priamry game logic ---------------------------------------------------------------
def beginGame(PLAYERCOLOR, PLAYERHOUSE, randomize):
    #Initialize settings
    header = pygame.font.Font("squealer.ttf", 20)
    text = pygame.font.Font("squealer.ttf", 15)
    screen= mechanics.screen

    mechanics.reset()    

    #If random game has been selected, randomly assign lands and player color
    if randomize:
      lands=[]
      for country in mechanics.countryStats:
        lands.append(country)
      colors=[]
      random.shuffle(lands)
      #Give each player one region
      for color in mechanics.players:
        if len(lands)>0:
          mechanics.countryStats[lands.pop()]= mechanics.Country(2, color)
          colors.append(color)
      #Apportion other lands as unowned
      for land in lands:
        owner= random.sample(colors,1)[0]
        mechanics.countryStats[land]= mechanics.Country(0,owner)
      #Assign the player to be a random house
      PLAYERCOLOR= random.sample(colors, 1)[0]
      PLAYERHOUSE= random.sample(HOUSES, 1)[0]
      if PLAYERCOLOR in mechanics.heraldry:
          if mechanics.heraldry[PLAYERCOLOR] in mechanics.redBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.redBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.orangeBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.orangeBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.yellowBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.yellowBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.greenBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.greenBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.blueBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.blueBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.purpleBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.purpleBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.whiteBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.whiteBlazons, 1)[0]
          elif mechanics.heraldry[PLAYERCOLOR] in mechanics.blackBlazons:
            mechanics.heraldry[PLAYERCOLOR]= random.sample(mechanics.blackBlazons, 1)[0]

    #Set player settings    
    mechanics.players[PLAYERCOLOR]= mechanics.Player(PLAYERHOUSE, PLAYERCOLOR)
    mechanics.PLAYERCOLOR= PLAYERCOLOR
    mechanics.PLAYERHOUSE= PLAYERHOUSE

    #primary game loop
    loops= 0
    peace= 0
    gameOver=False
    while 1:
        
      #Turn instantiations
      catch= True
      mechanics.allianceAsked=[]
      menu="none"
      spy="none"
      
      #Initial info display
      if loops==0:
        spy= (PLAYERCOLOR, "none")

      #Victory and loss condition checking
      if not PLAYERCOLOR in mechanics.players:
        if DEBUG: gameOver= True
        else: lose("")
      elif len(mechanics.players)==1: win("", loops)
      p= True
      for color in mechanics.players:
        if not color in mechanics.callBanners(PLAYERCOLOR):
          p=False
      if p: peace+=1
      else: peace=0
      if peace>=2*PEACE_TIME: win("Twenty year peace!", loops)
      
      #AI logic here -----------------------
      if loops>0:
        vassals= mechanics.callBanners(PLAYERCOLOR)
        for color in mechanics.players:
          if (not color in vassals) and (mechanics.getLiege(color)==color):
            mechanics.players[color].AI.computeOrders()
      #End AI logic here -------------------

      #Game logic
      mechanics.computeBribes()
      mechanics.executeOrders()
      mechanics.computeBuilding()
      mechanics.computeIncome()
      if (loops%2 == 0): mechanics.levyTroops()
      mechanics.computeSiege()
      mechanics.computeAssassinations()
      mechanics.computeAlliances(loops<1)

      #clears orders queue
      mechanics.clearOrders()
      #Displays the year
      if loops%2==0:
        mechanics.message= "Summer, "+str(1000+(loops/2))+" A.D."
      else:
        mechanics.message= "Winter, "+str(1000+(loops/2))+" A.D."

      #If in debugging mode, print player stats
      if DEBUG:
        for color in mechanics.players:
          print mechanics.players[color].toString()
        if gameOver:
          mechanics.infoMessages=[]
          mechanics.decisionQueue=[]

      loops=loops+1
      #wait for next turn
      print "---------------------------------------------"
      selected= "none"
      supporting= False
      messageBoxOpen= False
      decisionBoxOpen=False
      spyBoxOpen=False
      framecount= DEBUG_FRAMES
      #Poll until end of turn
      while catch:
        screen.fill(BLACK)
        screen.blit(mechanics.mapIm, (0,0))
        #Draw buttons
        screen.blit(mechanics.button, (10,550))
        display= header.render("End Turn", 1, TEXTCOL)
        screen.blit(display, (40,560))
        screen.blit(mechanics.button, (10,600))
        display= header.render("My House", 1, TEXTCOL)
        screen.blit(display, (40,610))
        screen.blit(mechanics.button, (10,650))
        display= header.render("Exit Game", 1, TEXTCOL)
        screen.blit(display, (40,660))
        #Draw all countries, their castles, and occupations
        for country in mechanics.countryStats:
          level=mechanics.countryStats[country].level
          owner=mechanics.countryStats[country].owner
          mechanics.drawCountry(country, owner, level)
        #draw all armies
        for army in mechanics.armies:
          mechanics.drawArmy(army.location, army.color)
        if selected!="none":
          mechanics.drawArmy(selected.location, "select")
          mechanics.drawArmy(selected.location,selected.color)
        #draw message
        display = header.render(mechanics.message, 1, TEXTCOL)
        screen.blit(display, (500,0))
        #Draw player info
        x,y=20,720
        if PLAYERCOLOR in mechanics.players:
          stringM= "House "+mechanics.players[PLAYERCOLOR].house
          stringM+=": "+str(int(mechanics.coffers[PLAYERCOLOR]))+" gold"
          display = text.render(stringM, 1, TEXTCOL)
          screen.blit(display, (x,y))
          vassals= mechanics.callBanners(PLAYERCOLOR)
          vassals.remove(PLAYERCOLOR)
          stringM=""
          if len(vassals)>0:
            stringM+="Commands: "
            for color in vassals:
              stringM+="House "+mechanics.players[color].house+", "
        else: stringM="Game Over"
        display = text.render(stringM, 1, TEXTCOL)
        screen.blit(display, (x,y+20))
        #Draw menu if necessary
        if menu!="none" and menu.color!="none":
          player= mechanics.players[menu.color]
          screen.blit(mechanics.menuBox, (menu.x, menu.y))
          houseCol= BLACK
          col= menu.color
          if col[0:len("red")]=="red": houseCol= RED
          elif col[0:len("orange")]=="orange": houseCol= ORANGE
          elif col[0:len("yellow")]=="yellow": houseCol= YELLOW
          elif col[0:len("green")]=="green": houseCol= GREEN
          elif col[0:len("blue")]=="blue": houseCol= BLUE
          elif col[0:len("purple")]=="purple": houseCol= PURPLE
          elif col[0:len("white")]=="white": houseCol= WHITE
          display= header.render("House "+player.house, 1, houseCol)
          screen.blit(display, (menu.x+15, menu.y+10))
          display= text.render(player.getLord(), 1, TEXTCOL)
          screen.blit(display, (menu.x+15, menu.y+35))
          display= text.render("Region: "+string.capwords(menu.country), 1, TEXTCOL)
          screen.blit(display, (menu.x+15, menu.y+60))
          #Menu options
          menuy= menu.y+110
          menuBounds={}
          menuOptions=["Send Spies", "Propose Alliance", "Grant Title", "Raze Buildings", "Burn Region","Bribe Castellan", "Assassinate "+player.title, "Assassinate Daughter"]
          for option in menuOptions:
            display= text.render(option, 1, TEXTCOL)
            screen.blit(display, (menu.x+15, menuy))
            menuBounds[option]=(menuy,menuy+20)
            menuy+=25
        #Draw spy screen if necessary
        if spy!="none" and not decisionBoxOpen and not messageBoxOpen:
          (col,loc)=spy
          mbX,mbY=200,200
          screen.blit(mechanics.infoBox, (mbX,mbY))
          player= mechanics.players[col]
          #Find House color
          houseCol= BLACK
          if col[0:len("red")]=="red": houseCol= RED
          elif col[0:len("orange")]=="orange": houseCol= ORANGE
          elif col[0:len("yellow")]=="yellow": houseCol= YELLOW
          elif col[0:len("green")]=="green": houseCol= GREEN
          elif col[0:len("blue")]=="blue": houseCol= BLUE
          elif col[0:len("purple")]=="purple": houseCol= PURPLE
          elif col[0:len("white")]=="white": houseCol= WHITE
          display= header.render("House "+player.house, 1, houseCol)
          screen.blit(display,(mbX+30, mbX+10))
          info= []
          info.append("under "+player.getLord()+", whom men call the "+player.monkier)
          #Find granted titles to
          grantedTitles="Titles to: "
          for land in player.titles:
            grantedTitles+=string.capwords(land)+", "
          if len(player.titles)>0: info.append(grantedTitles)
          #Find trait information
          max= player.maxAttribute()
          min= player.minAttribute()
          maxDesc=""
          minDesc=""
          if max=="honor": maxDesc="is honorable"
          if min=="honor": minDesc="is untrustworthy"
          if max=="greed": maxDesc="is greedy"
          if min=="greed": minDesc="is not posessed by greed"
          if max=="subtlety": maxDesc="is diplomatic"
          if min=="subtlety": minDesc="is tactless"
          if max=="command": maxDesc="is a decisive commander"
          if min=="command": minDesc="is a weak commander"
          if max=="dread": maxDesc="is dreaded by his subjects"
          if min=="dread": minDesc="is lenient with his subjects"
          info.append("The "+player.title+" "+maxDesc+", but he "+minDesc)
          #Find sworn houses
          info.append("")
          swornTo= "Sworn to House "+mechanics.players[mechanics.getLiege(col)].house
          if mechanics.getLiege(col)!=col: info.append(swornTo)
          swornHouses= "Sworn Houses: "
          if not col in mechanics.bannersDict: swornHouses+="none"
          elif len(mechanics.bannersDict[col])<=1: swornHouses+="none"
          else:
            for color in mechanics.bannersDict[col]:
              if color!=col: swornHouses+=mechanics.players[color].house+", "
          info.append(swornHouses)
          info.append("Number of marriageable daughters: "+str(player.daughters))
          #Find treasury info
          resources=""
          if col in mechanics.coffers: resources+="Treasury: "+str(int(mechanics.coffers[col]))+" gold pieces. "
          countries= mechanics.ownedCountries(col)
          income= 0
          for country in countries:
            income+=mechanics.countryStats[country].income
          resources+="Incomes of "+str(int(income*2))+" gold pieces per year."
          info.append(resources)
          if loc!="none":
            info.append(string.capwords(loc)+" produces "+str(int(mechanics.countryStats[loc].income*2))+" gold pieces per year.")
          messagey= mbY+35
          #Draw all information
          for message in info:
            display= text.render(message, 1, TEXTCOL)
            screen.blit(display, (mbX+30, messagey))
            messagey+=25
          spyBoxOpen=True
        #Draw decision box if necessary
        if len(mechanics.decisionQueue)>0 and not messageBoxOpen and not spyBoxOpen:
          dbX, dbY= 200, 200
          screen.blit(mechanics.decisionBox, (dbX, dbY))
          display=text.render(mechanics.decisionQueue[0].message, 1, TEXTCOL)
          screen.blit(display, (dbX+30,dbY+100))
          display=text.render(mechanics.decisionQueue[0].prompt, 1, TEXTCOL)
          screen.blit(display, (dbX+30,dbY+130))
          decisionBoxOpen=True
        #Draw info box if necessary
        if len(mechanics.infoMessages)>0 and not decisionBoxOpen and not spyBoxOpen:
          mbX,mbY=200,200
          screen.blit(mechanics.infoBox, (mbX,mbY))
          display = text.render(mechanics.infoMessages[0].message, 1, TEXTCOL)
          screen.blit(display, (mbX+30,mbY+100))
          messageBoxOpen=True
        #Push all screen updates
        pygame.display.flip()

        #If game is over in debugging mode, play out remainder
        if gameOver and DEBUG:
          framecount-=1
          catch= framecount>0

        #process events
        for event in pygame.event.get():
          if event.type == pygame.QUIT: sys.exit()
          if event.type == pygame.KEYDOWN:
            if event.unicode=="s" and not selected=="none":
              supporting= not supporting
              if supporting: mechanics.message= "...will support..."
              else: mechanics.message="Army at "+string.capwords(selected.location)+"..."
          if event.type == pygame.MOUSEBUTTONDOWN:
            (x,y)= event.pos
            button= event.button
            #A spy box is open
            if spyBoxOpen and button==1:
              if x>mbX and x<mbX+670 and y>mbY and y<mbY+292:
                spyBoxOpen= False
                spy="none"
            #A decision box is open
            elif decisionBoxOpen and button==1:
              if x>dbX+175 and x<dbX+290 and y>dbY+230 and y<dbY+280:
                mechanics.processDecision(mechanics.decisionQueue.pop(0), True)
              elif x>dbX+400 and x<dbX+480 and y>dbY+230 and y<dbY+280:
                mechanics.processDecision(mechanics.decisionQueue.pop(0), False)
              decisionBoxOpen=False
            #A message box is open
            elif messageBoxOpen and button==1:
              if x>mbX and x<mbX+670 and y>mbY and y<mbY+292:
                mechanics.infoMessages.pop(0)
                messageBoxOpen= False
            #Next turn button
            elif button==1 and x>10 and x<140 and y>550 and y<595:
              message=""
              catch= False
            #House info
            elif button==1 and x>10 and x<140 and y>600 and y<645 and not gameOver:
              spy=(PLAYERCOLOR, "none")
            #Exit button
            elif button==1 and x>10 and x<140 and y>650 and y<695:
              if DEBUG or WORK: quit()
              else: showMenu()
            #Right click to bring up a menu
            elif button==3 and not decisionBoxOpen and not messageBoxOpen and not gameOver:
              #Finds the closest country
              closestCountry= "None"
              minDist= float((2*width)**2)
              for country in mechanics.countryLocs:
                xC, yC= mechanics.countryLocs[country]
                dist= (x-(xC-5))**2 + (y-(yC+55))**2
                if dist<minDist:
                  minDist= dist
                  closestCountry= country
                  closestCountry= country
              owner= mechanics.countryStats[closestCountry].owner
              menu= Menu(owner, closestCountry, x, y)
            #Left click with menu open
            elif menu!="none" and button==1:
              if x<menu.x or x>menu.x+menu.width or y<menu.y or y>menu.y+menu.height:
                menu="none"
              else:
                for option in menuBounds:
                  (top,bottom)=menuBounds[option]
                  if y>top and y<bottom:
                    if option=="Bribe Castellan":
                      mechanics.bribeCastellan(menu.country, PLAYERCOLOR)
                    elif option=="Send Spies":
                      spy=(menu.color, menu.country)
                      mechanics.coffers[PLAYERCOLOR]-=SPYCOST
                    elif option=="Assassinate King" or option=="Assassinate Lord":
                      mechanics.processAssassin(menu.color, KING)
                    elif option=="Assassinate Daughter":
                      mechanics.processAssassin(menu.color, DAUGHTER)
                    elif option=="Raze Buildings":
                      if not mechanics.countryStats[menu.country].occupier in mechanics.callBanners(PLAYERCOLOR):
                        mechanics.message= "You do not occupy this region"
                      elif mechanics.countryStats[menu.country].level<1:
                        mechanics.message= "No buildings to raze"
                      else:
                        dString="This will destroy all buildings on "+string.capwords(menu.country)+"."
                        decision= mechanics.Decision(dString, "Do you wish to raze these buildings?", "raze", menu.country, menu.color)
                        mechanics.decisionQueue.append(decision)
                    elif option=="Burn Region":
                      if not mechanics.countryStats[menu.country].occupier in mechanics.callBanners(PLAYERCOLOR):
                        mechanics.message= "You do not occupy this region"
                      else:
                        dString="This will permanently decrease all incomes on "+string.capwords(menu.country)+"."
                        decision= mechanics.Decision(dString, "Do you wish to salt and burn this region?", "burn", menu.country,"")
                        mechanics.decisionQueue.append(decision)
                    elif option=="Propose Alliance":
                      if menu.color in mechanics.callBanners(PLAYERCOLOR):
                        mechanics.message= "You already command House "+mechanics.players[menu.color].house
                      elif len(mechanics.allianceAsked):
                        mechanics.message="Unable to propose another alliance this turn"
                      else:
                        dString="Do you wish to ask for allegiance from House "+mechanics.players[menu.color].house+"?"
                        decision= mechanics.Decision(dString, "", "alliance", PLAYERCOLOR, menu.color)
                        mechanics.decisionQueue.append(decision)
                        mechanics.allianceAsked.append(menu.color)
                    elif option=="Grant Title":
                      if not menu.color in mechanics.callBanners(PLAYERCOLOR):
                        mechanics.message= "You do not control "+string.capwords(menu.country)
                      else:
                        dString="Do you wish to grant "+string.capwords(menu.country)+", with all its lands and incomes,"
                        dString2="to House "+mechanics.players[menu.color].house+" ad infinitum?"
                        decision= mechanics.Decision(dString, dString2, "title", menu.color, menu.country)
                        mechanics.decisionQueue.append(decision)
                    menu="none"
            #Other left click
            elif button==1:
              #Finds the closest army
              closestArmy= "None"
              loc= 0,0
              minDist= float((2*width)**2)
              for country in mechanics.countryLocs:
                xA, yA= mechanics.countryLocs[country]
                dist= (x-(xA+45))**2 + (y-(yA-5))**2
                if dist<minDist:
                  minDist= dist
                  closestArmyLoc= country
                  loc= xA,yA
              xA, yA= loc
              closestArmy=""
              for army in mechanics.armies:
                if army.location==closestArmyLoc: closestArmy= army
              check = closestArmy!="" and closestArmy.color in mechanics.callBanners(PLAYERCOLOR)
              if x>xA+45 and x<xA+91 and y>yA-5 and y<yA+75 and selected=="none" and check:
                selected, supporting= mechanics.processArmy(closestArmy, closestArmyLoc, selected, supporting)
              elif selected!="none":
                selected, supporting= mechanics.processArmy(closestArmy, closestArmyLoc, selected, supporting)
              #Finds the closest country
              else:
                closestCountry= "None"
                minDist= float((2*width)**2)
                for country in mechanics.countryLocs:
                  xC, yC= mechanics.countryLocs[country]
                  dist= (x-(xC-5))**2 + (y-(yC+55))**2
                  if dist<minDist:
                    minDist= dist
                    closestCountry= country
                mechanics.processCountry(closestCountry)

#Show the splash screen
def showSplash():
  screen= mechanics.screen
  screen.fill(BLACK)
  screen.blit(mechanics.presentsIm, (0,0))
  pygame.display.flip()
  if WORK: time.sleep(.5)
  else: time.sleep(2)
  screen.fill(BLACK)
  screen.blit(mechanics.splashIm, (0,0))
  pygame.display.flip()
  if WORK: time.sleep(1)
  else: time.sleep(5)

#Exits the program
def quit():
  sys.exit(0)
  
#When any key is pressed, return to the main menu
def onClick(function):
  text = pygame.font.Font("squealer.ttf", 12)
  screen= mechanics.screen
  display= text.render("Press any key to continue...", 1, TEXTCOL)
  screen.blit(display, (420, 700))
  pygame.display.flip()
  time.sleep(0.25)
  while True:
    for event in pygame.event.get():
        if event.type == pygame.KEYDOWN or event.type==pygame.MOUSEBUTTONDOWN:
          function()
            
#Draws the winning screen
def win(message, loops):
  global SECRETS
  big = pygame.font.Font("squealer.ttf", 40)
  header = pygame.font.Font("squealer.ttf", 20)
  #Draw bg im
  screen= mechanics.screen
  screen.fill(BLACK)
  screen.blit(mechanics.infoScreen, (0,0))
  #Draw text
  display= big.render("Victory is Yours!", 1, TEXTCOL)
  screen.blit(display, (375, 300))
  display= header.render(message, 1, TEXTCOL)
  screen.blit(display, (420, 375))
  score= 1000-(loops/2) + mechanics.coffers[mechanics.PLAYERCOLOR]/1000
  if message=="Twenty year peace!": 
    SECRETS+=1
    score+= PEACE_TIME
  smessage= "Score: "+str(int(score))
  display= header.render(smessage, 1, TEXTCOL)
  screen.blit(display, (420, 450))
  smessage= "Secrets Discovered: "+str(SECRETS)+"/1"
  display= header.render(smessage, 1, TEXTCOL)
  screen.blit(display, (420, 525))
  pygame.display.flip()
  onClick(showMenu)

#Draws the losing screen
def lose(message):
  big = pygame.font.Font("squealer.ttf", 40)
  header = pygame.font.Font("squealer.ttf", 20)
  #Draw bg im
  screen= mechanics.screen
  screen.fill(BLACK)
  screen.blit(mechanics.infoScreen, (0,0))
  #Draw text
  display= big.render("You Have Been Defeated!", 1, TEXTCOL)
  screen.blit(display, (300, 300))
  display= header.render(message, 1, TEXTCOL)
  screen.blit(display, (420, 375))
  pygame.display.flip()
  onClick(showMenu)
  
#Display the game credits
def credits():
  header = pygame.font.Font("squealer.ttf", 15)
  #Draw bg im
  screen= mechanics.screen
  screen.fill(BLACK)
  screen.blit(mechanics.infoScreen, (0,0))
  #Draw text
  linesx= 100
  linesy=100
  credits=["Credits:", "  Project Founder: Caspar Anderegg", "  Lead Developer: Caspar Anderegg", "  Artistic Contributions From:", "    Phillip Capper"]
  for line in credits:
    display= header.render(line, 1, TEXTCOL)
    screen.blit(display, (linesx, linesy))
    linesy+=30
  pygame.display.flip()
  onClick(showMenu)
  
#Displays the main game menu
def showMenu():
  header = pygame.font.Font("squealer.ttf", 40)
  #Draw bg im
  screen= mechanics.screen
  screen.fill(BLACK)
  screen.blit(mechanics.menuIm, (0,0))
  #Draw menun options
  options=["New Game", "Random Game", "Credits", "Exit Game"]
  menuy= 718-50*len(options)
  menux= 75
  menuBounds={}
  for option in options:
    display= header.render(option, 1, WHITE)
    screen.blit(display, (menux, menuy))
    menuBounds[option]=(menuy,menuy+50)
    menuy+=50
  pygame.display.flip()
    
  #Poll keyboard
  while True:
    for event in pygame.event.get():
      if event.type==pygame.MOUSEBUTTONDOWN and event.button==1:
        (x,y)= event.pos
        for option in menuBounds:
          (top,bottom)=menuBounds[option]
          if y>top and y<bottom and x>menux and x<menux+200:
            if option=="New Game":
              beginGame("whiteandblue", "de Grey", False)
            elif option=="Random Game":
              beginGame("white", "", True)
            elif option=="Credits":
              credits()
            elif option=="Exit Game":
              quit()

#Program logic
if DEBUG: beginGame("whiteandblue", "de Grey", False)
showSplash()
showMenu()
