import random, sys
from constants import *

#Useful constants in the AI to use ------------------
BURN_BUFFER=3
GOLD_RESERVE=10000
DREAD_THRESHOLD=10
HONOR_THRESHOLD=10
COMMAND_THRESHOLD=15
fealtyAsked= False
#Helper methods for the AI to use -------------------

#Returns the players dictionary
def AIgetPlayers():
  return mechanics.players

#Returns the armies list
def AIgetArmies():
  return mechanics.armies

#Returns the countryStats dict
def AIgetCountries():
  return mechanics.countryStats

#Returns the coffers dict
def AIgetCoffers():
  return mechanics.coffers

#Returns a list of countries adjacent to the one specified
def adjacents(country):
  result=[]
  for country2 in AIgetCountries():
    if mechanics.adjacent(country, country2): result.append(country2)
  return result

#Returns any army of the given color threatening the given region, or "none" if there are none
def threatens(country, color):
  for army in AIgetArmies():
    if army.location==country and army.color==color: return army
  for army in AIgetArmies():
    if army.color==color and mechanics.adjacent(country, army.location):
      return army
  return "none"

#Returns true if an army is threatened, false otherwise
def isThreatened(army):
  team= mechanics.callBanners(mechanics.getOverlord(army.color))
  for color in AIgetPlayers():
    if not color in team and threatens(army.location, color)!="none":
      return True
  return False

#Counts the number of armies owned by a color
def countArmies(color):
  return mechanics.countArmies(color)

#Returns a list of countries owned by a color
def ownedCountries(color):
  return mechanics.ownedCountries(color)

#Returns a list of color's vassals (including color itself)
def callBanners(color):
  return mechanics.callBanners(color)

#Get the commanding house of color
def getOverlord(color):
  return mechanics.getOverlord(color)

#Gets a bribe price
def bribePrice(country):
  return mechanics.bribePrice(country)

#Returns true if there is an army of color on country
def armyOn(country, color):
  return mechanics.armyOn(country, color)

#An AI class ----------------------------------------
#Designed such that each player may have their own AI

#-----------------------------------------------------------
# AI Name: Hadrian
# version: 1.0
# Purpose: An AI bot capable of challenging a human player
# Author: Caspar J. Anderegg
# written 10 July 2011
#-----------------------------------------------------------
class Hadrian:
  #A list of possible trait goals for an AI
  __goals=["honor", "dread", "subtlety"]
  color=""
  goal=""
  countBurns=0
  
  #Creates a new AI with the specified color
  def __init__(self, color):
    self.color= color
    self.goal= random.sample(self.__goals, 1)[0]
    
  #Method called when a city is under siege
  def underSiege(self, country, occupier, remaining):
    prob= PROB_SURRENDER + (PROB_SURRENDER/10.0)*(AIgetPlayers()[occupier].honor - AIgetPlayers()[occupier].dread)
    lastStand= countArmies(self.color)<1 and len(ownedCountries(self.color))<2
    if remaining==1 and lastStand and not (self.color in callBanners(occupier)) and random.random()<prob:
      self.AIofferFealty(occupier)
      
  #Method called when a region is taken
  def regionTaken(self, country, agressor):
    castleTaken= AIgetCountries()[country].level>=2
    breach= agressor in callBanners(getOverlord(self.color))
    if castleTaken and breach:
      self.AIbreachOfTrust()
      
  #Method called when a region is burned
  def regionBurned(self, country, agressor):
    breach= agressor in callBanners(getOverlord(self.color))
    if breach: self.countBurns+=1
    if breach and self.goal=="dread": self.AIbreachOfTrust()
    elif self.countBurns>=BURN_BUFFER: self.AIbreachOfTrust()
    
  #Method called when a region is razed
  def regionRazed(self, country, agressor):
    breach= agressor in callBanners(getOverlord(self.color))
    if breach: self.AIbreachOfTrust()
    
  #Method called when a region is bribed
  def regionBribed(self, country, agressor):
    breach= agressor in callBanners(getOverlord(self.color))
    if breach: self.AIbreachOfTrust()
    
  #Sets the commands for a single turn
  def computeOrders(self):
    global fealtyAsked
    orderedArmies= []
    doNotMove=[]
    fealtyAsked= False
    
    #Propose alliances with neighbors
    vassals= callBanners(self.color)
    freeNeighbors=[]
    for country in AIgetCountries():
      if AIgetCountries()[country].owner in vassals:
        adj= adjacents(country)
        for ctry in adj:
          owner= AIgetCountries()[ctry].owner
          if not owner in vassals and getOverlord(owner)==owner and owner!=mechanics.PLAYERCOLOR:
            self.AIaskFealty(owner, AIgetPlayers()[self.color].daughters>0)
            
    #Grant all vassals titles to the lands they hold
    vassals= callBanners(self.color)
    for country in AIgetCountries():
      if AIgetCountries()[country].owner in vassals:
        self.AIgrantTitle(AIgetCountries()[country].owner, country)
        
    #Build up to the castle level as fast as possible
    for color in vassals:
      self.AIcastleRush(color)

    #If the goal is subtlety, send assassins after likely targets
    if self.goal=="subtlety":
      for color in AIgetPlayers():
        if not color in vassals:
          enemy= AIgetPlayers()[color]
          if enemy.honor>HONOR_THRESHOLD or enemy.dread>DREAD_THRESHOLD or enemy.command>COMMAND_THRESHOLD:
            self.AIsendAssassin(color, mechanics.KING)

    #Break all subject sieges you can
    underSiege=[]
    for country in AIgetCountries():
      countryObj= AIgetCountries()[country]
      if countryObj.owner!= countryObj.occupier and countryObj.owner in vassals:
        underSiege.append(country)
    for country in underSiege:
      region= AIgetCountries()[country]
      if not region.occupier in vassals:
        army= threatens(country, region.owner)
        if army!="none" and not army in orderedArmies and not country in doNotMove:
          self.AIorderArmy(army, country)
          orderedArmies.append(army)
          doNotMove.append(country)
      else:
        #If you want to exterminate the house, hold
        if self.goal=="dread" and ownedCountries(region.owner)<2 and region.owner!=self.color:
          army= threatens(country, region.occupier)
          if not army in OrderedArmies and not country in doNotMove:
            self.AIorderArmy(army, country)
            orderedArmies.append(army)
            doNotMove.append(country)
        #If you want need to back off, do that
        elif region.siege<=2 and (region.level>1 or ownedCountries(region.owner)<2 or region.owner==self.color):
          army= threatens(country, region.occupier)
          if not army in orderedArmies: 
            self.AIretreat(army)
            orderedArmies.append(army)

    #Hold any sieges that you currently have
    holdingSiege= []
    for country in AIgetCountries():
      region= AIgetCountries()[country]
      if region.occupier in vassals and not region.owner in vassals:
        holdingSiege.append(country)
    for country in holdingSiege:
      region= AIgetCountries()[country]
      #Raze if necessary
      if self.goal=="dread" or ownedCountries(region.occupier)<1:
        self.AIraze(country)
      #Hold position
      army= threatens(country, region.occupier)
      if not army in orderedArmies and not country in doNotMove:
        self.AIorderArmy(army, country)
        orderedArmies.append(army)
        doNotMove.append(country)

    #Move into unoccupied territories when doing so will not leave your own territory vulnerable
    for army in AIgetArmies():
      holdingCastle= AIgetCountries()[army.location].level>1
      if army.color in vassals and (not isThreatened(army) or holdingCastle):
        adj= adjacents(army.location)
        target="none"
        for country in adj:
          region= AIgetCountries()[country]
          if (not region.owner in vassals) and (not armyOn(country, region.occupier)):
            target= country
        if target!="none" and (not army in orderedArmies) and not (target in doNotMove):
          self.AIorderArmy(army, target)
          orderedArmies.append(army)
          doNotMove.append(target)

    #If you can move into another territory with support, try to do so
    for army in AIgetArmies():
      if army.color in vassals and not army in orderedArmies:
        for country in adjacents(army.location):
          region= AIgetCountries()[country]
          if not region.occupier in vassals and not country in doNotMove:
            support= []
            for a in AIgetArmies():
              if a.color in vassals and not a in orderedArmies and a.location in adjacents(country):
                support.append(a)
            if len(support)>0:
              self.AIorderArmy(army, country)
              orderedArmies.append(army)
              doNotMove.append(country)
              for a in support:
                self.AIorderArmy(a, army)
                orderedArmies.append(a)

    #If the goal is subtlety, bribe enemy castellans and move into the region in one turn
    if self.goal=="subtlety":
      available= []
      for army in AIgetArmies():
        if army.color==self.color and not army in orderedArmies:
          adj= adjacents(army.location)
          for country in adj:
            region= AIgetCountries()[country]
            if not region.owner in vassals and not army in available: available.append(army)
      for army in available:
        adj= adjacents(army.location)
        target= "none"
        for country in adj:
          region= AIgetCountries()[country]
          if not region.owner in vassals and not region.occupier in vassals and region.level>0:
            target=country
        if target!="none" and AIgetCoffers()[self.color]>bribePrice(target):
          self.AIbribe(target)
          if not army in orderedArmies and not target in doNotMove:
            self.AIorderArmy(army, target)
            orderedArmies.append(army)
            doNotMove.append(target)

    #Support randomly with any unordered armies
    #First round of support, support important things
    for army in AIgetArmies():
      if army.color in vassals and not army in orderedArmies:
        canSupport=[]
        for a in orderedArmies:
          orders= self.AIgetOrders(a)
          if orders in AIgetCountries() and orders in adjacents(army.location):
            canSupport.append(a)
        if len(canSupport)>0:
          self.AIorderArmy(army, canSupport[0])
          orderedArmies.append(army)            

    #Randomly order some threatened armies to hold
    NUM_TO_HOLD=2
    ordered=0
    for army in AIgetArmies():
      if army.color in vassals and not army in orderedArmies and isThreatened(army) and ordered<NUM_TO_HOLD:
        self.AIorderArmy(army, army.location)
        orderedArmies.append(army)
        doNotMove.append(army.location)
        ordered+=1

    #Support randomly with any unordered armies
    for army in AIgetArmies():
      if army.color in vassals and not army in orderedArmies:
        canSupport=[]
        for a in orderedArmies:
          orders= self.AIgetOrders(a)
          if orders in AIgetCountries() and orders in adjacents(army.location):
            canSupport.append(a)
        if len(canSupport)>0:
          self.AIorderArmy(army, canSupport[0])
          orderedArmies.append(army)

    #If any armies are *still* not ordered, move them randomly
    for army in AIgetArmies():
      if army.color in vassals and not army in orderedArmies:
        adj= adjacents(army.location)
        for a in AIgetArmies():
          if a.location in adj: adj.remove(a.location)
        for country in doNotMove:
          if country in adj: adj.remove(country)
        if len(adj)>0:
          self.AIorderArmy(army, adj[0])
          orderedArmies.append(army)
          doNotMove.append(adj[0])
        
    #If gold left over, build up to citadel level
    for color in vassals:
      self.AIcitadelReserve(color)

    #If goal is subtlety, assassinate daughters of rival houses, keeping GOLD_RESERVE in reserve
    if self.goal=="subtlety":
      for color in AIgetPlayers():
        if not color in vassals and AIgetCoffers()[self.color]>=GOLD_RESERVE+ASSASSIN_COST:
          self.AIsendAssassin(color, mechanics.DAUGHTER)
      
  #HELPER METHODS FOR THE AI TO USE---------------------------------------------------------------------------------
  #DO NOTE EDIT METHODS BELOW THIS POINT
  #Offer fealty. Will be accepted unless declined by the player 
  def AIofferFealty(self, overColor):
    if overColor in callBanners(mechanics.PLAYERCOLOR):
      dString= "House "+AIgetPlayers()[self.color].house+" wishes to swear allegiance to House "+AIgetPlayers()[overColor].house
      decision= mechanics.Decision(dString, "Will you allow it?", "fealty", overColor, self.color)
      mechanics.decisionQueue.append(decision)
    else:
      mechanics.swearBanner(overColor, self.color)
      AIgetPlayers()[self.color].AI.countBurns=0
      
  #Declare a breach of trust and rebel
  def AIbreachOfTrust(self):
    liege= getOverlord(self.color)
    mechanics.freeBanner(self.color)
    if countArmies(self.color)>0 and len(ownedCountries(self.color))>0:
      stringM= "Breach of trust, House "+AIgetPlayers()[self.color].house+" has declared themselves free."
      print stringM
      mechanics.infoMessages.append(mechanics.Message(stringM, "rebellion"))
      
  #Ask for fealty from a non-PLAYERCOLOR player
  def AIaskFealty(self, underColor, daughter):
    global fealtyAsked
    if fealtyAsked: return
    if getOverlord(underColor)!=underColor: return
    if underColor==mechanics.PLAYERCOLOR: return
    if daughter and AIgetPlayers()[self.color].daughters<=0: return
    mechanics.considerAlliance(self.color, underColor, daughter)
    fealtyAsked= True
    
  #Grants a title to some vassal
  def AIgrantTitle(self, vassal, country):
    if not vassal in callBanners(self.color): return
    if AIgetCountries()[country].owner!=vassal: return
    if country in AIgetPlayers()[vassal].titles: return
    mechanics.grantTitle(vassal, country)
    
  #Sends an assassin
  def AIsendAssassin(self, toColor, target):
    if AIgetCoffers()[self.color]<mechanics.ASSASSIN_COST: return
    mechanics.sendAssassin(self.color, toColor, target)
    
  #Order an army to move or support as directed
  def AIorderArmy(self, army, arg2):
    if not army.color in callBanners(self.color): return
    mechanics.orderArmy(army,arg2)
    
  #Tries to move this army into an unoccupied territory
  #If no unoccupied territories, will hold position
  def AIretreat(self, army):
    if not army.color in callBanners(self.color): return
    adj= adjacents(army.location)
    team= callBanners(getOverlord(self.color))
    for army in AIgetArmies():
      if army.location in adj: adj.remove(army.location)
    #Try to retreat to your own provinces
    for location in adj:
      if AIgetCountries()[location].owner==self.color:
        self.AIorderArmy(army, location)
        return
    #Try to retreat to friendly provinces
    for location in adj:
      if AIgetCountries()[location].owner in team:
        self.AIorderArmy(army, location)
        return
    #Try to retreat to any provinces
    for location in adj:
      self.AIorderArmy(army, location)
    #Hold position
    self.AIorderArmy(army, army.location)
    
  #Build in a rush up to the castle level
  def AIcastleRush(self, color):
    if not color in callBanners(self.color): return
    for country in AIgetCountries():
      region= AIgetCountries()[country]
      if region.owner==color and region.level<2:
        mechanics.build(country)
        
  #Build citadels, keeping GOLD_RESERVE gold in reserve
  def AIcitadelReserve(self, color):
    if not color in callBanners(self.color): return
    for country in AIgetCountries():
      region= AIgetCountries()[country]
      afford= color in AIgetCoffers() and AIgetCoffers()[color]>=GOLD_RESERVE+CITYCOST
      if region.owner==color and region.level<2 and afford:
        mechanics.build(country)

  #Raze a building
  def AIraze(self, country):
    vassals= callBanners(self.color)
    if not AIgetCountries()[country].occupier in vassals: return
    mechanics.raze(country)
    AIgetPlayers()[self.color].honor-=1

  #Bribe the castellan on a country
  def AIbribe(self, country):
    mechanics.bribeCastellan(country, self.color)

  #Find the orders associated with an army, or "none" if none
  def AIgetOrders(self, army):
    if not army.color in callBanners(self.color): return "none"
    if not army in mechanics.ordersQueue: return "none"
    return mechanics.ordersQueue[army]

import mechanics

